//*****************************************
// Graphic LCD Serial(RS232) Terminal Program ATMega88
// CPU Clock:O9.216MHz(9.22)i3.3VPOlgȉȂ̂ł̂܂܂Ŏgj
//   iq[Yf[^@WȂŁAOUWlgȏɂj
//
//@kbc͂PQWSWhbgOtBbNW[gp
//    AQM1248A-RN
//
//@ȉkbcɕKvȐMꗗ
//
//  PC0: LCD-CS*
//  PC1: LCD-RS
//  PC2: LCD-SCLK
//  PC3: LCD-SDA
//  PC5: LCD-RST*
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2014/03/28 New
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>
#include	<avr/pgmspace.h>

#include	"micro_terminal.h"

#include	"Pfont5x7.h"
#include	"LCD.h"


unsigned char	disp_moji[43];															// kbc\

// ******** MACRO ********
#define	rst_on()				(PORTC &= ~(1 << PC5))						// Reset* Sig:Low
#define	rst_off()				(PORTC |=  (1 << PC5))						// Reset* Sig:High
#define	select_instruction()	(PORTC &= ~(1 << PC1))						// D/I* Sig:Low
#define	select_data()			(PORTC |=  (1 << PC1))						// D/I* Sig:High
#define	cs_on()					(PORTC &= ~(1 << PC0))						// CS Sig:Low
#define	cs_off()				(PORTC |=  (1 << PC0))						// CS Sig:Hogh
#define	sclk_h()				(PORTC |=  (1 << PC2))						// SCLK:High
#define	sclk_l()				(PORTC &= ~(1 << PC2))						// SCLK:Low
#define	sda_h()					(PORTC |=  (1 << PC3))						// SDA:High
#define	sda_l()					(PORTC &= ~(1 << PC3))						// SDA:Low

// ************************************* Graphic LCD Accese *********************
static	unsigned char	c_data0[10];


//******************************
// Get Character Bit Data
// 5x7 Font
//  Get Data Save to c_data0[5];
//******************************
static void	get_asc_data(unsigned char uc)
{
	unsigned short	us;

	us = uc;

	c_data0[0] = pgm_read_byte(&(font5x7[us * 5 + 0]));
	c_data0[1] = pgm_read_byte(&(font5x7[us * 5 + 1]));
	c_data0[2] = pgm_read_byte(&(font5x7[us * 5 + 2]));
	c_data0[3] = pgm_read_byte(&(font5x7[us * 5 + 3]));
	c_data0[4] = pgm_read_byte(&(font5x7[us * 5 + 4]));
}

//******************************
// LCD Bit Data Output
//  uc : Output Data(D0 -- D7)
//
//  PC0: LCD-CS*
//  PC1: LCD-RS
//  PC2: LCD-SCLK
//  PC3: LCD-SDA
//  PC5: LCD-RST*
//
//  \ qrAbr̓ZbgKvL
// *****************************
static void	lcd_G_bitdata_out(unsigned char uc)
{
	unsigned char	i;

	for(i=0x80;i != 0;i >>= 1){				// 8Bit Output
		if((uc & i)==0){					// Out Bit:0(ZERO)
			sda_l();						// SDA:Low
		}else{
			sda_h();						// SDA:High
		}
		sclk_l();							// SCLK:Low
		sclk_h();							// SCLK:High
	}
}
//******************************
// LCD Data Output
//  uc : Output Data(D0 -- D7)
//  rs : RS bit Data(0=Command, 1=Data)
//
//  PC0: LCD-CS*
//  PC1: LCD-RS
//  PC2: LCD-SCLK
//  PC3: LCD-SDA
//  PC5: LCD-RST*
//
// *****************************
static void	lcd_G_one_out(unsigned char uc, int rs)
{
	cs_on();								// CS:ON

	if(rs == 0){							// RS:0(Command)
		select_instruction();				// D/I* Sig:Low
	}else{									// RS:1(Data)
		select_data();						// D/I* Sig:High
	}

	lcd_G_bitdata_out(uc);					// 8Bit Output

	cs_off();								// CS:OFF
}
//******************************
// LCD Command Output
//  uc : Command Data(D0 -- D7)
// *****************************
static	void	lcd_G_cmd_out(unsigned char uc)
{
	lcd_G_one_out(uc, 0);
}
//******************************
// LCD Data Output
//  uc : Data(D0 -- D7)
// *****************************
static	void	lcd_G_data_out(unsigned char uc)
{
	lcd_G_one_out(uc, 1);
}
//******************************
// Init. LCD
//
//  PC0: LCD-CS*
//  PC1: LCD-RS
//  PC2: LCD-SCLK
//  PC3: LCD-SDA
//  PC5: LCD-RST*
//
// *****************************
void	init_lcd_G(void)
{
	cs_off();																	// CS:OFF
	sclk_h();																	// SCLK:High
	DDRC |= ((1 << DDC0)|(1 << DDC1)|(1 << DDC2)|(1 << DDC3)|(1 << DDC5));		// Port-C Bit0--3 & Bit5 : Output

	rst_on();																	// Reset* Sig:Low
	delay_ms(100);																// 1mS
	rst_off();																	// Reset* Sig:High

	delay_ms(100);

	lcd_G_cmd_out(0xAE);														// Display OFF
	lcd_G_cmd_out(0xA0);														// ADC Normal
	lcd_G_cmd_out(0xC8);														// Common Output:revers
	lcd_G_cmd_out(0xA3);														// bias:1/7

	// Internal Regulator ON
	lcd_G_cmd_out(0x2C);														// Power Control 1
	delay_ms(10);
	lcd_G_cmd_out(0x2E);														// Power Control 2
	delay_ms(10);
	lcd_G_cmd_out(0x2F);														// Power Control 3
	delay_ms(10);

	// Contrast
	lcd_G_cmd_out(0x23);														// Vo Voltage Resistor Ratio set
	lcd_G_cmd_out(0x81);														// Electrpnic Volume Mode set
	lcd_G_cmd_out(0x1C);														// Electrpnic Volume Value set

	// Display
	lcd_G_cmd_out(0xA4);														// Display All Ponts:normal
	lcd_G_cmd_out(0x40);														// Display Start Line:0(ZERO)
	lcd_G_cmd_out(0xA6);														// Display Nomal/Revers:nomal
	lcd_G_cmd_out(0xAF);														// Display ON

}
//******************************
// Clear Display
// *****************************
void	disp_clear_lcd_G(void)
{
	unsigned char	x;
	unsigned char	y;


	for(y=0;y < 8;y++){
		lcd_G_cmd_out((0xb0 | y));	// Page Set
		lcd_G_cmd_out(0x10);		// Column High Address Set
		lcd_G_cmd_out(0x00);		// Column Low Address Set
		for(x=0;x <128; x++){
			lcd_G_data_out(0x00);
		}
	}
}
//******************************
// One Line Clear Display
//   y: line number(0--7)
// *****************************
void	disp_line_clear_lcd_G(unsigned char y)
{
	unsigned char	x;

	lcd_G_cmd_out((0xb0 | y));	// Page Set
	lcd_G_cmd_out(0x10);		// Column High Address Set
	lcd_G_cmd_out(0x00);		// Column Low Address Set
	for(x=0;x <128; x++){
		lcd_G_data_out(0x00);
	}
}
//******************************
// display LCD One charcter
//  pos_x : Position X(0 -- 20)
//  pos_y : Position Y(0 -- 7)
//  uc : Charcter
//    E\sb`͂UhbgitHgԂPhbgLj
// *****************************
void	disp_one_char_lcd_G(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	pos_x *= 6;
	get_asc_data(uc);						// Get Character Bit Data
	lcd_G_cmd_out((0xb0 | pos_y));			// Page Set
	lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
	lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
	lcd_G_data_out(c_data0[0]);
	lcd_G_data_out(c_data0[1]);
	lcd_G_data_out(c_data0[2]);
	lcd_G_data_out(c_data0[3]);
	lcd_G_data_out(c_data0[4]);
	lcd_G_data_out(0x00);
}
//******************************
// display LCD One charcteri]\j
//  pos_x : Position X(0 -- 20)
//  pos_y : Position Y(0 -- 7)
//  uc : Charcter
//    E\sb`͂UhbgitHgԂPhbgLj
// *****************************
void	disp_one_char_lcd_G_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	pos_x *= 6;
	get_asc_data(uc);						// Get Character Bit Data
	lcd_G_cmd_out((0xb0 | pos_y));			// Page Set
	lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
	lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
	lcd_G_data_out(c_data0[0] ^ 0xff);
	lcd_G_data_out(c_data0[1] ^ 0xff);
	lcd_G_data_out(c_data0[2] ^ 0xff);
	lcd_G_data_out(c_data0[3] ^ 0xff);
	lcd_G_data_out(c_data0[4] ^ 0xff);
	lcd_G_data_out(0xff);
}
//******************************
// display LCD(20char x 8Line)
//  pos_x : Position X(0 -- 20)
//  pos_y : Position Y(0 -- 7,\̂ 0 -- 5)
//  ucp : String(termnate 0x00)
// *****************************
void	disp_lcd_G(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos = pos_x;
	while(*ucp != 0x00){				// String Data Out
		if(20 < pos){
			break;
		}
		disp_one_char_lcd_G(pos, pos_y, *ucp);
		ucp++;
		pos++;
	}
}
//******************************
// display LCD(20char x 8Line, ]\)
//  pos_x : Position X(0 -- 20)
//  pos_y : Position Y(0 -- 7,\̂ 0 -- 5)
//  ucp : String(termnate 0x00)
// *****************************
void	disp_lcd_G_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos = pos_x;
	while(*ucp != 0x00){				// String Data Out
		if(20 < pos){
			break;
		}
		disp_one_char_lcd_G_reverse(pos, pos_y, *ucp);
		ucp++;
		pos++;
	}
}

// tHg`FbN\
//******************************
// display Font LCD(21char x 6Line)
//  Code 0 --> 120
// *****************************
void	disp_lcd_G_font_check1(void)
{
	unsigned char	x;
	unsigned char	y;
	unsigned char	uc;

	uc = 0;
	for(y=0;y<6;y++){
		for(x=0;x<21;x++){
			disp_one_char_lcd_G(x, y, uc);
			uc++;
		}
	}
}

//******************************
// display Font LCD(21char x 6Line)
//  Code 121 --> 241
// *****************************
void	disp_lcd_G_font_check2(void)
{
	unsigned char	x;
	unsigned char	y;
	unsigned char	uc;

	uc = 121;
	for(y=0;y<6;y++){
		for(x=0;x<21;x++){
			disp_one_char_lcd_G(x, y, uc);
			uc++;
		}
	}
}

//******************************
// display Font LCD(20char x 6Line)
//  Code 242 --> 255
// *****************************
void	disp_lcd_G_font_check3(void)
{
	unsigned char	x;
	unsigned char	y;
	unsigned char	uc;

	uc = 240;
	for(y=0;y<1;y++){
		for(x=0;x<14;x++){
			disp_one_char_lcd_G(x, y, uc);
			uc++;
		}
	}
}
