//*****************************************
// Graphic LCD Serial(RS232) Terminal Program ATMega88
// CPU Clock:O9.216MHz(9.22)i3.3VPOlgȉȂ̂ł̂܂܂Ŏgj
//   iq[Yf[^@WȂŁAOUWlgȏɂj
//
//
//@LED1:DS1(PC4): 1=Light ON, 0=Light OFF
//@LED2:DS2(PD6): 1=Light ON, 0=Light OFF
//@LED4:DS4(PD5): 1=Light ON, 0=Light OFF
//@LED5:DS5(PD4): 1=Light ON, 0=Light OFF
//
//  2014/03/28 New
//*****************************************


#include	<avr/interrupt.h>
#include	<avr/io.h>
#include	<avr/pgmspace.h>
#include	<avr/sleep.h>
#include	<avr/eeprom.h>
#include	<string.h>
#include	<stdio.h>

#include	"micro_terminal.h"
#include	"uart.h"
#include	"LCD.h"
#include	"Timer.h"
#include	"switch.h"

static	unsigned char	char_list_nomal[N_of_SEL_CHAR_LINE][N_of_SEL_CHAR_COLUMN]={
	//0    1    2   3    4   5   6   7   8   9  10  11  12  13  14  15  16  17   18  19  20  
	{'a', 'b', 'c','d', 'e','f','g','h','i','j','k','l','m','n','o','p','q','r', 's','t','u',},
	{'v', 'w', 'x','y', 'z','0','1','2','3','4','5','6','7','8','9',' ','!',0x22,'#','$','%',},
	{0x0d,0x0a,'&',0x27,'(',')','*','+',',','-','.','/',':',';','<','>','?','[', ']','^','_',},
};	// VtgȂ̑I𕶎
static	unsigned char	char_list_sift[N_of_SEL_CHAR_LINE][N_of_SEL_CHAR_COLUMN]={	// Vtg̑I𕶎
	//0    1    2   3    4   5   6   7   8   9  10  11  12  13  14  15  16  17   18  19  20  
	{'A', 'B', 'C','D', 'E','F','G','H','I','J','K','L','M','N','O','P','Q','R', 'S','T','U',},
	{'V', 'W', 'X','Y', 'Z','0','1','2','3','4','5','6','7','8','9',' ','!',0x22,'#','$','%',},
	{0x0d,0x0a,'&',0x27,'(',')','*','+',',','-','.','/',':',';','<','>','?','[', ']','^','_',},
};	// Vtg̑I𕶎
static	unsigned char	rx_disp_data[N_of_RX_DISP_LINE][N_of_RX_DISP_COLUMN];		// Mf[^

// **********************************************************
// ********************** Soft Timer ************************
// **********************************************************

volatile unsigned int	wait_i;
volatile unsigned int	wait_j;
volatile unsigned char	wait_us;
// ************************************
// Delay 2uS
// t: uS (max 255)
// CPU Clock:9.216MHz
// ************************************
void delay_2us(unsigned char t)
{
	unsigned char	i;

	for(i=0;i<t;i++){
		wait_us++;				// 1uS Delay??
		wait_j++;				// 1uS Delay??
	}
}
//***************************
// Wait
//  tm : Wait Time(ms) max. 8000
//***************************
void	delay_ms(int tm)
{

	for(wait_i = 0; wait_i < tm; wait_i++){
		delay_2us(500/2);
		delay_2us(500/2);
	}
}

//***************************
// \f[^Psɂ炷
// ԉ̍s̓NA
//***************************
static
void	moveup_disp_data(void)
{
	unsigned char	x;
	unsigned char	y;

	for(y=1;y<=MAX_DISP_POS_Y;y++){
		for(x=0;x<=MAX_DISP_POS_X;x++){
			rx_disp_data[y-1][x] = rx_disp_data[y][x];
		}
	}

	for(x=0;x<=MAX_DISP_POS_X;x++){
		rx_disp_data[MAX_DISP_POS_Y][x] = ' ';
	}
}
//***************************
// M\f[^NA
//***************************
static
void	rx_data_clr(void)
{
	unsigned char	x;
	unsigned char	y;

	for(y=0;y<=MAX_DISP_POS_Y;y++){
		for(x=0;x<=MAX_DISP_POS_X;x++){
			rx_disp_data[y][x] = ' ';
		}
	}
}
//***************************
// Mf[^Sĕ\
//***************************
static
void	disp_rx_disp_data_all(void)
{
	unsigned char	y;

	for(y=0;y<=MAX_DISP_POS_Y;y++){
		disp_lcd_G(0, DISP_RXD_OFS+y, &rx_disp_data[y][0]);
	}
}
//***************************
// VtgȂ̑I𕶎Sĕ\
//***************************
static
void	disp_char_data_all(void)
{
	unsigned char	y;

	for(y=0;y<=MAX_SEL_CHAR_POS_Y;y++){
		disp_lcd_G(0, DISP_CHAR_OFS+y, &char_list_nomal[y][0]);
	}
}
//***************************
//  Main
//***************************
int	main(void)
{

	unsigned char	i;
	char			uc;
	unsigned char	sel_char_pos_x=0;
	unsigned char	sel_char_pos_y=0;
	unsigned char	disp_pos_x=0;
	unsigned char	disp_pos_y=0;

	uart_init();												// UART Initial

	DDRC = (1 << DDC4);
	PORTC &= ~(1 << PC4);

	init_lcd_G();
	disp_clear_lcd_G();
	sw_init();													//	SW PORT(PB) ݒ

	disp_lcd_G( 0, 0,(unsigned char *)"--   AT MEGA88    --");				// LCD Disolay String
	disp_lcd_G( 0, 1,(unsigned char *)"-- Micro Terminal --");		// LCD Disolay String
	disp_lcd_G( 0, 2,(unsigned char *)Version);					// LCD Disolay String
	disp_lcd_G( 0, 4,(unsigned char *)(__DATE__));				// LCD Disolay String
	disp_lcd_G( 12, 4,(unsigned char *)(__TIME__));				// LCD Disolay String

	delay_ms(10000);

	disp_clear_lcd_G();
	Timer2_init();												// Timer2 Initial
	set_sleep_mode(SLEEP_MODE_IDLE);							// X[vFACh[h

	rx_data_clr();												// M\f[^NA
	disp_rx_disp_data_all();									// Mf[^Sĕ\
	disp_char_data_all();										// VtgȂ̑I𕶎Sĕ\
	disp_one_char_lcd_G_reverse(sel_char_pos_x, DISP_CHAR_OFS+sel_char_pos_y, char_list_nomal[sel_char_pos_y][sel_char_pos_x]);	// I𕶎

	sei();														// 荞݉
	for(i=0;;i++){

		// XCb`͏
		if(gOn_Right == ON){									// Right Switch
			gOn_Right = OFF;
			disp_one_char_lcd_G(sel_char_pos_x, DISP_CHAR_OFS+sel_char_pos_y, char_list_nomal[sel_char_pos_y][sel_char_pos_x]);	// I𕶎
			if(gIn_Sift == 0){									// Right
				if(MAX_SEL_CHAR_POS_X == sel_char_pos_x){		// Eς̎
					sel_char_pos_x = 0;
					if(MAX_SEL_CHAR_POS_Y == sel_char_pos_y){	// ς̎
						sel_char_pos_y = 0;
					}else{
						sel_char_pos_y++;
					}
				}else{
					sel_char_pos_x++;
				}
			}else{												// Down
				if(MAX_SEL_CHAR_POS_Y == sel_char_pos_y){	// ς̎
					sel_char_pos_y = 0;
				}else{
					sel_char_pos_y++;
				}
			}
			disp_one_char_lcd_G_reverse(sel_char_pos_x, DISP_CHAR_OFS+sel_char_pos_y, char_list_nomal[sel_char_pos_y][sel_char_pos_x]);	// I𕶎
		}
		if(gOn_Left == ON){										// Left Switch
			gOn_Left = OFF;
			disp_one_char_lcd_G(sel_char_pos_x, DISP_CHAR_OFS+sel_char_pos_y, char_list_nomal[sel_char_pos_y][sel_char_pos_x]);	// I𕶎
			if(gIn_Sift == 0){									// Right
				if(0 == sel_char_pos_x){						// ς̎
					sel_char_pos_x = MAX_SEL_CHAR_POS_X;
					if(0 == sel_char_pos_y){					// アς̎
						sel_char_pos_y = MAX_SEL_CHAR_POS_Y;
					}else{
						sel_char_pos_y--;
					}
				}else{
					sel_char_pos_x--;
				}
			}else{												// Up
				if(0 == sel_char_pos_y){						// アς̎
					sel_char_pos_y = MAX_SEL_CHAR_POS_Y;
				}else{
					sel_char_pos_y--;
				}
			}
			disp_one_char_lcd_G_reverse(sel_char_pos_x, DISP_CHAR_OFS+sel_char_pos_y, char_list_nomal[sel_char_pos_y][sel_char_pos_x]);	// I𕶎
		}
		if(gOn_Enter == ON){									// Enter
			gOn_Enter = OFF;
			if(gIn_Sift == 0){									// Vtg̎
				uc = char_list_nomal[sel_char_pos_y][sel_char_pos_x];
			}else{												// VtgL̎
				uc = char_list_sift[sel_char_pos_y][sel_char_pos_x];
			}
			PORTD |= (1 << PD6);
			tim_10ms_led2 = 10;									// kdcQpPOrPʃ^C}[
			uart_tx_one(uc);									// UART TxD One Char
		}

		// Mf[^
		if(Get_rxd((unsigned char *)(&uc)) != 0){				// M
			PORTC |= (1 << PC4);
			tim_10ms_led1 = 10;									// kdcPpPOrPʃ^C}[
			if(uc == 0x0d){										// CR
				disp_one_char_lcd_G(disp_pos_x, DISP_RXD_OFS+disp_pos_y, ' ');	// ܂ł̃J[\
				disp_pos_x = 0;
			}else if(uc == 0x0a){								// LF
				if(MAX_DISP_POS_Y == disp_pos_y){				// ς̎
					moveup_disp_data();							// \f[^Psɂ炷
					disp_rx_disp_data_all();					// Mf[^Sĕ\
				}else{
					disp_one_char_lcd_G(disp_pos_x, DISP_RXD_OFS+disp_pos_y, ' ');	// ܂ł̃J[\
					disp_pos_y++;
				}
			}else{
				rx_disp_data[disp_pos_y][disp_pos_x] = uc;
				if(disp_pos_x == MAX_DISP_POS_X){				// Eς̎
					if(MAX_DISP_POS_Y == disp_pos_y){			// ς̎
						moveup_disp_data();						// \f[^Psɂ炷
						disp_rx_disp_data_all();				// Mf[^Sĕ\
					}else{
						disp_one_char_lcd_G(disp_pos_x, DISP_RXD_OFS+disp_pos_y, uc);	// ܂ł̃J[\
						disp_pos_y++;
					}
					disp_pos_x = 0;
				}else{
					disp_one_char_lcd_G(disp_pos_x, DISP_RXD_OFS+disp_pos_y, uc);	// ܂ł̃J[\
					disp_pos_x++;
				}
			}
			disp_one_char_lcd_G(disp_pos_x, DISP_RXD_OFS+disp_pos_y, '_');	// J[\
		}

		if(tim_10ms_led1 == 0){									// kdcPpPOrPʃ^C}[
			PORTC &= ~(1 << PC4);
		}
		if(tim_10ms_led2 == 0){									// kdcQpPOrPʃ^C}[
			PORTD &= ~(1 << PD6);
		}

		sleep_mode();											// X[vڍs
	}

	while(1){
		;
	}
}
